from textual_autocomplete._autocomplete import (
    AutoComplete,
    AutoCompleteList,
    DropdownItem,
    DropdownItemHit,
    TargetState,
)

from textual_autocomplete._path_autocomplete import PathAutoComplete

__all__ = [
    "AutoComplete",
    "PathAutoComplete",
    "AutoCompleteList",
    "DropdownItem",
    "DropdownItemHit",
    "TargetState",
]



"""Basic dropdown autocomplete from a list of options."""

from textual.app import App, ComposeResult
from textual.widgets import Input

from textual_autocomplete import AutoComplete

LANGUAGES = [
    "Python",
    "JavaScript",
    "TypeScript",
    "Java",
    "C++",
    "Ruby",
    "Go",
    "Rust",
]


class AutoCompleteExample(App[None]):
    def compose(self) -> ComposeResult:
        text_input = Input(placeholder="Search for a programming language...")
        yield text_input

        yield AutoComplete(
            target=text_input,  # The widget to attach autocomplete to
            candidates=LANGUAGES,  # The list of completion candidates
        )


if __name__ == "__main__":
    app = AutoCompleteExample()
    app.run()



from textual.app import App, ComposeResult
from textual.content import Content
from textual.widgets import Input

from textual_autocomplete import AutoComplete, DropdownItem


LANGUAGES = [
    DropdownItem(
        "Python",
        prefix=Content.from_markup("[$text-success on $success-muted] 🐍 "),
    ),
    DropdownItem(
        "Golang",
        prefix=Content.from_markup("[$text-primary on $primary-muted] 🔷 "),
    ),
    DropdownItem("Java", prefix=Content.from_markup("[#6a2db5 on magenta 20%] ☕ ")),
    DropdownItem(
        "Rust", prefix=Content.from_markup("[$text-accent on $accent-muted] 🦀 ")
    ),
]


class LanguagesSearchApp(App[None]):
    CSS = """
    Input {
        margin: 2 4;
    }
    """

    def compose(self) -> ComposeResult:
        input_widget = Input(placeholder="Search for a programming language...")
        yield input_widget
        yield AutoComplete(target=input_widget, candidates=LANGUAGES)


if __name__ == "__main__":
    app = LanguagesSearchApp()
    app.run()



"""A two-column autocomplete example with heavy styling."""

from textual.app import App, ComposeResult
from textual.content import Content
from textual.widgets import Input, Label

from textual_autocomplete import AutoComplete, DropdownItem
from examples._headers import headers

# Define a mapping of sections to colors
SECTION_COLORS: dict[str, str] = {
    "Authentication": "$text-success",
    "Caching": "$text-primary",
    "Conditionals": "$text-warning",
    "Connection management": "$text-error",
    "Content negotiation": "$text-success",
    "Controls": "$text-accent",
    "Cookies": "$text-warning",
    "CORS": "$text-error",
    # Add fallback color for any other sections
    "default": "$foreground",
}

# Create dropdown items with two columns: rank and language name
CANDIDATES = [
    DropdownItem(
        Content.styled(
            str(header["name"]),
            style=SECTION_COLORS.get(
                str(header.get("section", "default")), SECTION_COLORS["default"]
            ),
        ),  # Main text to be completed with color based on section
        prefix=Content.from_markup(
            "[$text-primary on $primary-muted] $number", number=f"{i:<3}"
        ),  # Prefix showing rank, styled with Textual markup!
    )
    for i, header in enumerate(headers)
]


class TwoColumnAutoCompleteExample(App[None]):
    def compose(self) -> ComposeResult:
        yield Label("Start typing an HTTP header name:")
        text_input = Input(placeholder="Type here...")
        yield text_input

        yield AutoComplete(
            target=text_input,  # The widget to attach autocomplete to
            candidates=CANDIDATES,  # The list of completion candidates
        )


if __name__ == "__main__":
    app = TwoColumnAutoCompleteExample()
    app.run()



"""Let's ensure the `prevent_default_tab` and `prevent_default_enter` options work as expected."""

from __future__ import annotations

from typing import Any
from textual.app import App, ComposeResult
from textual.widgets import Button, Input
from textual_autocomplete import AutoComplete


class PreventDefaultTab(App[None]):
    """Test that the default tab key is permitted if `prevent_default_tab` is `False`."""

    def __init__(
        self, prevent_default_tab: bool = False, *args: Any, **kwargs: Any
    ) -> None:
        super().__init__(*args, **kwargs)
        self.prevent_default_tab = prevent_default_tab

    def compose(self) -> ComposeResult:
        input = Input(placeholder="Type something...")
        yield input
        yield AutoComplete(
            input,
            candidates=["foo", "bar"],
            prevent_default_tab=self.prevent_default_tab,
        )
        yield Button("I'm here to test focus.")


async def test_switch_focus_on_completion_via_tab_key__prevent_default_tab_is_false():
    """The default tab key should not be prevented if `prevent_default_tab` is `False`."""
    app = PreventDefaultTab(prevent_default_tab=False)
    async with app.run_test() as pilot:
        await pilot.press("f", "tab")
        assert app.query_one(Input).value == "foo"
        assert app.query_one(Button).has_focus


async def test_no_focus_switch_via_tab_key__prevent_default_tab_is_true():
    """`prevent_default_tab` is `True`, so focus should switch on completion with tab."""
    app = PreventDefaultTab(prevent_default_tab=True)
    async with app.run_test() as pilot:
        await pilot.press("f", "tab")
        input_widget = app.query_one(Input)
        assert input_widget.value == "foo"
        assert input_widget.has_focus
        assert not app.query_one(Button).has_focus



from __future__ import annotations

from textual.app import App, ComposeResult
from textual.containers import Center, VerticalScroll
from textual.pilot import Pilot
from textual.widgets import Input
from textual_autocomplete import AutoComplete


class CursorTracking(App[None]):
    CSS = """
    #scrollable {
        overflow: scroll scroll;
        scrollbar-color: red;
        Center {
            background: $accent;
            height: 100;
            width: 100;
            align-vertical: middle;
        }
        Input {
            width: 24;
        }
    }
    """

    def compose(self) -> ComposeResult:
        with VerticalScroll(id="scrollable", can_focus=False):
            with Center():
                yield (input := Input(placeholder="Type here..."))

        yield AutoComplete(
            target=input,
            candidates=["foo", "bar", "baz", "qux", "boop"],
        )


def test_dropdown_tracks_terminal_cursor_when_parent_scrolls(snap_compare):
    """We type, the dropdown appears, then we scroll the parent container so that the position of the input
    and the dropdown changes on screen. The dropdown should remain aligned to the Input widget."""

    async def run_before(pilot: Pilot[None]) -> None:
        await pilot.press("b")
        scrollable = pilot.app.query_one("#scrollable")
        scrollable.scroll_relative(x=5, y=5, animate=False, force=True)
        await pilot.pause()

    assert snap_compare(CursorTracking(), run_before=run_before)


def test_dropdown_tracks_input_cursor_and_cursor_prefix_as_search_string(snap_compare):
    """The completions should be based on the text between the start of the input and the cursor location.

    In this example, we type "ba", then move the cursor back one cell so that the search string is "b",
    meaning the completions should be "bar", "baz", and "boop".
    """

    async def run_before(pilot: Pilot[None]) -> None:
        await pilot.press(*"ba")  # Type "ba"
        await pilot.press("left")  # Move the cursor back one cell

    assert snap_compare(CursorTracking(), run_before=run_before)


def test_dropdown_tracks_input_cursor_on_click_and_cursor_prefix_search_string(
    snap_compare,
):
    """The completions should be based on the text between the start of the input and the cursor location.

    In this example, we type "ba", then move the cursor back one cell by clicking, so that the search string is "b",
    meaning the completions should be "bar", "baz", and "boop".
    """

    async def run_before(pilot: Pilot[None]) -> None:
        await pilot.press(*"ba")  # Type "ba"
        input_widget = pilot.app.query_one(Input)
        await pilot.click(input_widget, offset=(4, 1))  # Click on the "a"

    assert snap_compare(CursorTracking(), run_before=run_before)



from textual.app import App, ComposeResult
from textual.widgets import Input

from textual_autocomplete import AutoComplete
from textual_autocomplete._autocomplete import DropdownItem, TargetState


class DynamicDataApp(App[None]):
    CSS = """
Input {
    margin: 2 4;
}
"""

    def compose(self) -> ComposeResult:
        input_widget = Input()
        yield input_widget
        yield AutoComplete(input_widget, candidates=self.get_candidates)

    def get_candidates(self, state: TargetState) -> list[DropdownItem]:
        left = len(state.text)
        return [
            DropdownItem(item, prefix=f"{left:>2} ")
            for item in [
                "Apple",
                "Banana",
                "Cherry",
                "Orange",
                "Pineapple",
                "Strawberry",
                "Watermelon",
            ]
        ]


if __name__ == "__main__":
    app = DynamicDataApp()
    app.run()



from __future__ import annotations

from textual.app import App, ComposeResult
from textual.pilot import Pilot
from textual.widgets import Input
from textual_autocomplete import AutoComplete


class StyledAutocomplete(App[None]):
    def compose(self) -> ComposeResult:
        input = Input(placeholder="Search...")
        yield input
        yield AutoComplete(target=input, candidates=["foo", "bar", "baz", "qux"])


def test_foreground_color_and_text_style(snap_compare):
    """Background color should not be impacted by the text foreground and style."""
    StyledAutocomplete.CSS = """
    AutoComplete {
        & .autocomplete--highlight-match {
            color: $text-accent;
            text-style: bold italic underline;
        }
    }
    """

    async def run_before(pilot: Pilot) -> None:
        await pilot.press(*"ba")

    assert snap_compare(StyledAutocomplete(), run_before=run_before)


def test_background_color_and_removed_style(snap_compare):
    StyledAutocomplete.CSS = """
    AutoComplete {
        & .autocomplete--highlight-match {
            color: $text-accent;
            background: $success-muted;
            text-style: not bold;
        }
    }
    """

    async def run_before(pilot: Pilot) -> None:
        await pilot.press(*"ba")

    assert snap_compare(StyledAutocomplete(), run_before=run_before)


def test_max_height_and_scrolling(snap_compare):
    """We should be scrolled to qux, and the red scrollbar should reflect that."""
    StyledAutocomplete.CSS = """
    AutoComplete {
        & AutoCompleteList {
            scrollbar-color: red;
            max-height: 2;
        }
    }
    """

    async def run_before(pilot: Pilot) -> None:
        await pilot.press("down", "down", "down", "down")

    assert snap_compare(StyledAutocomplete(), run_before=run_before)


def test_dropdown_styles_match_textual_theme(snap_compare):
    """Dropdown styles should match the textual theme. In this test, we've swapped the theme to nord."""
    StyledAutocomplete.CSS = """
    AutoComplete {
        & .autocomplete--highlight-match {
            color: $text-accent;
        }
    }
    """

    async def run_before(pilot: Pilot) -> None:
        pilot.app.theme = "nord"
        await pilot.press(*"ba")

    assert snap_compare(StyledAutocomplete(), run_before=run_before)


def test_cursor_color_change_and_dropdown_background_change(snap_compare):
    """Checking various interactions between styles. See the test's CSS for info."""
    StyledAutocomplete.CSS = """
    AutoComplete {

        & AutoCompleteList {
            color: red;
            background: $error-muted;
            & > .option-list--option-highlighted {
                color: $text-primary;
                background: magenta;
                text-style: italic;
            }
        }

        & .autocomplete--highlight-match {
            color: $text-success;
            background: $success-muted;
            text-style: underline;
        }

    }
    """

    async def run_before(pilot: Pilot) -> None:
        await pilot.press(*"ba")

    assert snap_compare(StyledAutocomplete(), run_before=run_before)



from __future__ import annotations

import os
from pathlib import Path
from typing import Any, Callable
from os import DirEntry
from textual.content import Content
from textual.widgets import Input
from textual.cache import LRUCache

from textual_autocomplete import DropdownItem, AutoComplete, TargetState


class PathDropdownItem(DropdownItem):
    def __init__(self, completion: str, path: Path) -> None:
        super().__init__(completion)
        self.path = path


def default_path_input_sort_key(item: PathDropdownItem) -> tuple[bool, bool, str]:
    """Sort key function for results within the dropdown.

    Args:
        item: The PathDropdownItem to get a sort key for.

    Returns:
        A tuple of (is_dotfile, is_file, lowercase_name) for sorting.
    """
    name = item.path.name
    is_dotfile = name.startswith(".")
    return (not item.path.is_dir(), not is_dotfile, name.lower())


class PathAutoComplete(AutoComplete):
    def __init__(
        self,
        target: Input | str,
        path: str | Path = ".",
        *,
        show_dotfiles: bool = True,
        sort_key: Callable[[PathDropdownItem], Any] = default_path_input_sort_key,
        folder_prefix: Content = Content("📂"),
        file_prefix: Content = Content("📄"),
        prevent_default_enter: bool = True,
        prevent_default_tab: bool = True,
        cache_size: int = 100,
        name: str | None = None,
        id: str | None = None,
        classes: str | None = None,
        disabled: bool = False,
    ) -> None:
        """An autocomplete widget for filesystem paths.

        Args:
            target: The target input widget to autocomplete.
            path: The base path to autocomplete from.
            show_dotfiles: Whether to show dotfiles (files/dirs starting with ".").
            sort_key: Function to sort the dropdown items.
            folder_prefix: The prefix for folder items (e.g. 📂).
            file_prefix: The prefix for file items (e.g. 📄).
            prevent_default_enter: Whether to prevent the default enter behavior.
            prevent_default_tab: Whether to prevent the default tab behavior.
            cache_size: The number of directories to cache.
            name: The name of the widget.
            id: The DOM node id of the widget.
            classes: The CSS classes of the widget.
            disabled: Whether the widget is disabled.
        """
        super().__init__(
            target,
            None,
            prevent_default_enter=prevent_default_enter,
            prevent_default_tab=prevent_default_tab,
            name=name,
            id=id,
            classes=classes,
            disabled=disabled,
        )
        self.path = Path(path) if isinstance(path, str) else path
        self.show_dotfiles = show_dotfiles
        self.sort_key = sort_key
        self.folder_prefix = folder_prefix
        self.file_prefix = file_prefix
        self._directory_cache: LRUCache[str, list[DirEntry[str]]] = LRUCache(cache_size)

    def get_candidates(self, target_state: TargetState) -> list[DropdownItem]:
        """Get the candidates for the current path segment.

        This is called each time the input changes or the cursor position changes/
        """
        current_input = target_state.text[: target_state.cursor_position]

        if "/" in current_input:
            last_slash_index = current_input.rindex("/")
            path_segment = current_input[:last_slash_index] or "/"
            directory = self.path / path_segment if path_segment != "/" else self.path
        else:
            directory = self.path

        # Use the directory path as the cache key
        cache_key = str(directory)
        cached_entries = self._directory_cache.get(cache_key)

        if cached_entries is not None:
            entries = cached_entries
        else:
            try:
                entries = list(os.scandir(directory))
                self._directory_cache[cache_key] = entries
            except OSError:
                return []

        results: list[PathDropdownItem] = []
        for entry in entries:
            # Only include the entry name, not the full path
            completion = entry.name
            if not self.show_dotfiles and completion.startswith("."):
                continue
            if entry.is_dir():
                completion += "/"
            results.append(PathDropdownItem(completion, path=Path(entry.path)))

        results.sort(key=self.sort_key)
        folder_prefix = self.folder_prefix
        file_prefix = self.file_prefix
        return [
            DropdownItem(
                item.main,
                prefix=folder_prefix if item.path.is_dir() else file_prefix,
            )
            for item in results
        ]

    def get_search_string(self, target_state: TargetState) -> str:
        """Return only the current path segment for searching in the dropdown."""
        current_input = target_state.text[: target_state.cursor_position]

        if "/" in current_input:
            last_slash_index = current_input.rindex("/")
            search_string = current_input[last_slash_index + 1 :]
            return search_string
        else:
            return current_input

    def apply_completion(self, value: str, state: TargetState) -> None:
        """Apply the completion by replacing only the current path segment."""
        target = self.target
        current_input = state.text
        cursor_position = state.cursor_position

        # There's a slash before the cursor, so we only want to replace
        # the text after the last slash with the selected value
        try:
            replace_start_index = current_input.rindex("/", 0, cursor_position)
        except ValueError:
            # No slashes, so we do a full replacement
            new_value = value
            new_cursor_position = len(value)
        else:
            # Keep everything before and including the slash before the cursor.
            path_prefix = current_input[: replace_start_index + 1]
            new_value = path_prefix + value
            new_cursor_position = len(path_prefix) + len(value)

        with self.prevent(Input.Changed):
            target.value = new_value
            target.cursor_position = new_cursor_position

    def post_completion(self) -> None:
        if not self.target.value.endswith("/"):
            self.action_hide()

    def should_show_dropdown(self, search_string: str) -> bool:
        default_behavior = super().should_show_dropdown(search_string)
        return (
            default_behavior
            or (search_string == "" and self.target.value != "")
            and self.option_list.option_count > 1
        )

    def clear_directory_cache(self) -> None:
        """Clear the directory cache. If you know that the contents of the directory have changed,
        you can call this method to invalidate the cache.
        """
        self._directory_cache.clear()
        target_state = self._get_target_state()
        self._rebuild_options(target_state, self.get_search_string(target_state))



"""Snapshot tests for the Input widget with autocomplete."""

from __future__ import annotations

from textual.app import App, ComposeResult
from textual.pilot import Pilot
from textual.widgets import Input
from textual.widgets.input import Selection

from textual_autocomplete import AutoComplete, AutoCompleteList, DropdownItem

LANGUAGES = [
    "Python",
    "JavaScript",
    "TypeScript",
    "Java",
    "C++",
    "Ruby",
    "Go",
    "Rust",
    "C#",
    "Swift",
    "Kotlin",
    "PHP",
]
CANDIDATES = [DropdownItem(lang) for lang in LANGUAGES]


class BasicInputAutocomplete(App[None]):
    def compose(self) -> ComposeResult:
        input = Input(placeholder="Type here...")
        yield input
        yield AutoComplete(
            target=input,
            candidates=CANDIDATES,
        )
        yield Input(placeholder="Another input which can be focused")


def test_single_matching_candidate(snap_compare):
    """Typing should make the dropdown appear and show filtered results."""

    async def run_before(pilot: Pilot[None]) -> None:
        await pilot.press(*"py")

    assert snap_compare(BasicInputAutocomplete(), run_before=run_before)


def test_many_matching_candidates(snap_compare):
    """Typing should make the dropdown appear and show filtered results."""

    async def run_before(pilot: Pilot[None]) -> None:
        await pilot.press(*"ja")

    assert snap_compare(BasicInputAutocomplete(), run_before=run_before)


def test_selecting_candidate_should_complete_input__enter_key(snap_compare):
    """Selecting a candidate using the enter key should complete the input."""

    async def run_before(pilot: Pilot[None]) -> None:
        await pilot.press(*"ja")
        await pilot.press("down")
        await pilot.press("enter")

    assert snap_compare(BasicInputAutocomplete(), run_before=run_before)


def test_selecting_candidate_should_complete_input__tab_key(snap_compare):
    """Selecting a candidate using the tab key should complete the input."""

    async def run_before(pilot: Pilot[None]) -> None:
        await pilot.press(*"ja")
        await pilot.press("down")
        await pilot.press("tab")

    assert snap_compare(BasicInputAutocomplete(), run_before=run_before)


def test_tab_still_works_after_completion(snap_compare):
    """Tabbing after completing an input should still work (the autocomplete should not consume the tab key).

    The second input should become focused in this example."""

    async def run_before(pilot: Pilot[None]) -> None:
        await pilot.press(*"ja")
        await pilot.press("down")
        await pilot.press("tab")
        await pilot.press("tab")

    assert snap_compare(BasicInputAutocomplete(), run_before=run_before)


def test_summon_by_pressing_down(snap_compare):
    """We can summon the autocomplete dropdown by pressing the down arrow key."""

    async def run_before(pilot: Pilot[None]) -> None:
        await pilot.press("down")

    assert snap_compare(BasicInputAutocomplete(), run_before=run_before)


def test_summon_by_pressing_down_after_performing_completion(snap_compare):
    """We can summon the autocomplete dropdown by pressing the down arrow key,
    and it should be filled based on the current content of the Input.

    In this example, when we resummon the dropdown, the highlighted text should
    be "Java" - NOT "ja". "Java" is the current content of the input. "ja" was
    the text we previously performed the completion with.

    There was a bug where the dropdown would contain the pre-completion candidates.
    """

    async def run_before(pilot: Pilot[None]) -> None:
        await pilot.press(*"ja")  # Filters down to 2 candidates: JavaScript and Java
        await pilot.press("down")  # Move cursor over Java.
        await pilot.press("tab")  # Press tab to complete.
        await pilot.press("down")  # Press down to summon the dropdown.

    assert snap_compare(BasicInputAutocomplete(), run_before=run_before)


def test_hide_after_summoning_by_pressing_escape(snap_compare):
    """Dropdown summoned via down, then escape was pressed to hide it."""

    async def run_before(pilot: Pilot[None]) -> None:
        await pilot.press("down")
        await pilot.press("escape")

    assert snap_compare(BasicInputAutocomplete(), run_before=run_before)


def test_summon_when_only_one_full_match_does_not_show_dropdown(snap_compare):
    """If the dropdown contains only one item, and that item is an exact match for the dropdown
    content, then the dropdown should not be shown."""

    async def run_before(pilot: Pilot[None]) -> None:
        await pilot.press(*"py")
        await pilot.press("enter")
        await pilot.press("down")

    assert snap_compare(BasicInputAutocomplete(), run_before=run_before)


def test_hide_after_typing_by_pressing_escape(snap_compare):
    """Dropdown summoned via typing a matching query, then escape was pressed to hide it."""

    async def run_before(pilot: Pilot[None]) -> None:
        await pilot.press(*"py")
        await pilot.press("escape")

    assert snap_compare(BasicInputAutocomplete(), run_before=run_before)


def test_candidate_can_be_selected_via_click(snap_compare):
    """A candidate can be selected by clicking on it.

    In this example, we click on "Java", which is the second result in the dropdown.
    """

    async def run_before(pilot: Pilot[None]) -> None:
        await pilot.press(*"ja")
        await pilot.click(AutoCompleteList, offset=(1, 1))  # Click second result

    assert snap_compare(BasicInputAutocomplete(), run_before=run_before)


def test_text_selection_works_while_autocomplete_is_open(snap_compare):
    """If the dropdown is open, the text selection should still work."""

    async def run_before(pilot: Pilot[None]) -> None:
        await pilot.press(*"ja")
        input = pilot.app.query_one(Input)
        input.selection = Selection(0, 2)

    assert snap_compare(BasicInputAutocomplete(), run_before=run_before)


def test_completion_still_works_if_chosen_while_input_widget_has_selection(
    snap_compare,
):
    """If the dropdown is open, and a candidate is chosen, the completion should still
    work, and the selection should move to the end of the input."""

    async def run_before(pilot: Pilot[None]) -> None:
        await pilot.press(*"ja")
        input = pilot.app.query_one(Input)
        input.selection = Selection(0, 2)
        await pilot.press("down")
        await pilot.press("enter")

    assert snap_compare(BasicInputAutocomplete(), run_before=run_before)


def test_dropdown_tracks_cursor_position(snap_compare):
    """The dropdown should track the cursor position of the target widget."""

    async def run_before(pilot: Pilot[None]) -> None:
        await pilot.press(*"ja")
        await pilot.press("down")


def test_multiple_autocomplete_dropdowns_on_a_single_input(snap_compare):
    """Multiple autocomplete dropdowns can be open at the same time on a single input.

    I'm not sure why you'd want to do this. The behaviour is kind of undefined - both
    dropdowns should appear, but they'll overlap. Let's just ensure we don't crash.
    """

    class MultipleAutocompleteDropdowns(App[None]):
        def compose(self) -> ComposeResult:
            yield (input1 := Input(placeholder="Type here..."))
            yield AutoComplete(target=input1, candidates=LANGUAGES)
            yield AutoComplete(
                target=input1,
                candidates=["foo", "bar", "java", "javas", "javassss", "jajaja"],
            )

    async def run_before(pilot: Pilot[None]) -> None:
        await pilot.press("tab")
        await pilot.press(*"ja")
        await pilot.press("down")

    assert snap_compare(MultipleAutocompleteDropdowns(), run_before=run_before)


def test_multiple_autocomplete_dropdowns_on_same_screen(snap_compare):
    """Multiple autocomplete dropdowns can exist on the same screen."""

    class MultipleAutocompleteDropdowns(App[None]):
        def compose(self) -> ComposeResult:
            yield (input1 := Input(placeholder="Type here..."))
            # Setup with strings...
            yield AutoComplete(target=input1, candidates=LANGUAGES)
            yield (input2 := Input(placeholder="Also type here..."))
            # ...and with DropdownItems...
            yield AutoComplete(target=input2, candidates=CANDIDATES)

    async def run_before(pilot: Pilot[None]) -> None:
        await pilot.press("tab")
        await pilot.press(*"ja")
        await pilot.press("down")

    assert snap_compare(MultipleAutocompleteDropdowns(), run_before=run_before)



"""Two column dropdown example with simple styling on the prefix."""

from textual.app import App, ComposeResult
from textual.content import Content
from textual.widgets import Input, Label

from textual_autocomplete import AutoComplete, DropdownItem

# Languages with their popularity rank
LANGUAGES_WITH_RANK = [
    (1, "Python"),
    (2, "JavaScript"),
    (3, "Java"),
    (4, "C++"),
    (5, "TypeScript"),
    (6, "Go"),
    (7, "Ruby"),
    (8, "Rust"),
]

# Create dropdown items with two columns: rank and language name
CANDIDATES = [
    DropdownItem(
        language,  # Main text to be completed
        prefix=Content.from_markup(
            f"[$text-primary on $primary-muted] {rank} "
        ),  # Prefix showing rank, styled with Textual markup!
    )
    for rank, language in LANGUAGES_WITH_RANK
]


class TwoColumnAutoCompleteExample(App[None]):
    def compose(self) -> ComposeResult:
        yield Label("Start typing a programming language:")
        text_input = Input(placeholder="Type here...")
        yield text_input

        yield AutoComplete(
            target=text_input,  # The widget to attach autocomplete to
            candidates=CANDIDATES,  # The list of completion candidates
        )


if __name__ == "__main__":
    app = TwoColumnAutoCompleteExample()
    app.run()



"""By default, textual-autocomplete replaces the entire content of a widget
with a completion.

Sometimes, however, you may wish to insert the completion text, or otherwise use
custom logic to determine the end-state of the Input after the user selects a completion.

For example, if completing a path on a filesystem, you may wish to offer partial completions
of the path based on the current content of the Input. Then, when the user selects a completion,
you could offer a different set of completions based on the new path in the Input.
"""

from textual.app import App, ComposeResult
from textual.containers import Container
from textual.widgets import Button, Input, Label

from textual_autocomplete import PathAutoComplete


class FileSystemPathCompletions(App[None]):
    CSS = """
    #container {
        align: center middle;
        padding: 2 4;
    }
    #label {
        margin-left: 3;
    }
    Input {
        width: 80%;
    }
    """

    def compose(self) -> ComposeResult:
        with Container(id="container"):
            yield Label("Choose a file!", id="label")
            input_widget = Input(placeholder="Enter a path...")
            yield input_widget
        yield PathAutoComplete(target=input_widget, path="../textual")


if __name__ == "__main__":
    app = FileSystemPathCompletions()
    app.run()
